# -----------------------------------------------------------------------------
# Imports
# -----------------------------------------------------------------------------

from typing import Any, Dict, Optional

from pydantic import Field, root_validator

from acamodels.aca_base import ACABase

# -----------------------------------------------------------------------------
# Model
# -----------------------------------------------------------------------------


class Identification(ACABase):
    """File identification datamodel."""

    puid: Optional[str] = Field(...)
    signature: Optional[str] = Field(...)
    warning: Optional[str]

    @root_validator
    def check_puid_sig(cls, fields: Dict[Any, Any]) -> Dict[Any, Any]:
        """Validate that a PUID cannot have an empty signature
        or vice versa."""

        puid = fields.get("puid")
        signature = fields.get("signature")

        if puid is not None and signature is None:
            raise ValueError(f"Signature missing for PUID {puid}.")

        if signature is not None and puid is None:
            raise ValueError(f"PUID missing for signature {signature}.")

        return fields



# -----------------------------------------------------------------------------
# Imports
# -----------------------------------------------------------------------------

from pathlib import Path
from typing import Optional

from pydantic import validator

from acamodels._internals import size_fmt
from acamodels.aca_base import ACABase
from acamodels.identification import Identification

# -----------------------------------------------------------------------------
# Model
# -----------------------------------------------------------------------------


class ArchiveFile(ACABase):
    """ArchiveFile data model."""

    path: Path
    checksum: Optional[str]
    identification: Optional[Identification]

    # Validators
    @validator("path")
    def path_must_be_file(cls, path: Path) -> Path:
        """Resolves the file path and validates that it points
        to an existing file."""
        if not path.resolve().is_file():
            raise ValueError("File does not exist")
        return path.resolve()

    # Methods
    def read_text(self) -> str:
        """Expose read_text() functionality from pathlib.
        Encoding is set to UTF-8.

        Returns
        -------
        str
            File text data.
        """
        return self.path.read_text(encoding="utf-8")

    def read_bytes(self) -> bytes:
        """Expose read_bytes() functionality from pathlib.

        Returns
        -------
        bytes
            File byte data.
        """
        return self.path.read_bytes()

    def name(self) -> str:
        """Get the file name.

        Returns
        -------
        str
            File name.
        """
        return self.path.name

    def ext(self) -> str:
        """Get the file extension.

        Returns
        -------
        str
            File extension.
        """
        return self.path.suffix.lower()

    def size(self) -> str:
        """Get the file size in human readable string format.

        Returns
        -------
        str
            File size in human readable format.
        """
        return size_fmt(self.path.stat().st_size)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['acamodels']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.6,<2.0']

setup_kwargs = {
    'name': 'acamodels',
    'version': '0.5.0',
    'description': 'Datamodels based on pydantic used in Python tools developed at Aarhus City Archives.',
    'long_description': '[![Aarhus Stadsarkiv](https://raw.githubusercontent.com/aarhusstadsarkiv/py-template/master/img/logo.png)](https://www.aarhusstadsarkiv.dk/)\n# datamodels [![codecov](https://codecov.io/gh/aarhusstadsarkiv/datamodels/branch/master/graph/badge.svg)](https://codecov.io/gh/aarhusstadsarkiv/datamodels) [![Tests](https://github.com/aarhusstadsarkiv/datamodels/workflows/Tests/badge.svg)](https://github.com/aarhusstadsarkiv/datamodels/actions?query=workflow%3ATests)\n\n\nDatamodels based on [pydantic](https://github.com/samuelcolvin/pydantic/) used in Python tools developed at Aarhus City Archives.\n\n#### Structure\nEach model is placed in a separate `.py` file in order to achieve maintainability and better version control. In addition, each model must be served in `__init__.py` such that it is possible to call `from acamodels import model`.\n\n#### Versioning\n- **Updating** one or more models is considered a **patch** version, e.g. `0.1.0 -> 0.1.1`\n- **Adding** new models is considered a **minor** version, e.g. `0.1.0 -> 0.2.0`\n\nMajor versions will be pushed when models have reached a yet to be determined mature stage.\n',
    'author': 'Nina Jensen',
    'author_email': 'jnik@aarhus.dk',
    'maintainer': 'Aarhus Stadsarkiv',
    'maintainer_email': 'stadsarkiv@aarhus.dk',
    'url': 'https://www.aarhusstadsarkiv.dk/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
